#-------------------------------------------------------------------------------
# FIGURE 3
#-------------------------------------------------------------------------------

# R version 3.5.3 (2019-03-11) -- "Great Truth"
# Platform: x86_64-w64-mingw32/x64 (64-bit)

source("../../2-experiments/experimental-functions.R")
library(virtualNicheR) # version 1.0
library(MASS) # version 7.3-51.1
library(fields) # version 9.8-3
library(raster) # version 3.3-7
library(extrafont) # version 0.17
# For Windows - adjust the path to match your installation of Ghostscript
Sys.setenv(R_GSCMD = "C:/Program Files/gs/gs9.25/bin/gswin64c.exe")
set.seed(123) # set random seed for reproducable results

#-------------------------------------------------------------------------------

# Create coordinates across niche space
dim = 121
xMin = -6
xMax = 18
yMin = 0
yMax = 6800
niche.XY = niche.grid.coords(mins=c(xMin, yMin), maxs=c(xMax, yMax), nCoords=dim)
gridCoordsX = unique(niche.XY[,1])
gridCoordsY = unique(niche.XY[,2])

#-------------------------------------------------------------------------------

# Get the virtual species
species = opaleye()
lambdaMax = species[[1]]

# Calculate the fundamental niche
fundNiche = fund.niche(niche.XY, species)
fundNicheMatrix = matrix(fundNiche, nrow=length(unique(niche.XY[,1])))

#-------------------------------------------------------------------------------

# Read in raster climate data
nz.temp = raster("../../1-data/temp.tif")
nz.rain = raster("../../1-data/rain.tif")
rain = na.omit(values(nz.rain))
temp = na.omit(values(nz.temp))

#-------------------------------------------------------------------------------
 
# Calculate or read in NZ niche space density
if(!file.exists("nz-climate-space.txt")) {
  # Calculate climate density
  kde = kde2d(x=temp, y=rain, n = dim,
              lims = c(xMin, xMax, yMin, yMax))
  nz.climate = kde$z
  write.matrix(x = nz.climate, file="nz-climate-space.txt")
} else {
  nz.climate = matrix(scan("nz-climate-space.txt"), 
                      ncol=dim, nrow=dim, byrow = TRUE)
}

#-------------------------------------------------------------------------------

pdf("figure3.pdf", family="CM Roman", width=15/2.54, height=18/2.54, pointsize = 8)
par(mfrow=c(3,3), oma=c(8,3,0,0.5), mar=c(0,1,2,0.5), mgp=c(2,0.5,0), cex=1, tcl=-0.25)

# Examples of varying sample size
for (m in c(40, 100, 160)) {

  detections.XY = generateSpeciesObservations(species, m)
  
  nContour = 5
  cols = colorRampPalette(c("gray90", "gray40"))(nContour)

  image(unique(niche.XY[,1]), unique(niche.XY[,2]), fundNicheMatrix,
        xlim=c(xMin,xMax), xaxp=c(xMin,xMax,3), ylim=c(yMin,yMax), yaxp=c(yMin,yMax,5),
        breaks = seq(0, lambdaMax, lambdaMax/nContour), col=cols, main="",
        bty="n", xaxs="i", yaxs="i", xaxt="n", yaxt="n")
  contour(x=gridCoordsX, y=gridCoordsY, z=nz.climate, 
          levels = c(0.0000001, 0.0000005, 0.000001, 0.000005, 0.00001, 0.00005, 0.0001),
          col="black", lwd=0.25, drawlabels=FALSE, add=TRUE)
  points(detections.XY, pch=21, bg="#2c7bb6", cex=0.75, lwd=0.1)
  axis(1, at = seq(xMin, xMax, 6), labels = FALSE)
  
  if (m == 40) {
    axis(2, seq(yMin, yMax, 1700), las=1)
    mtext(expression(paste("(a) ", italic(m), " = 40")), at=6, side=3, cex=1, las=1, line=0)
  } 
  if (m == 100) {
    axis(2, seq(yMin, yMax, 1700), las=1, labels = FALSE)
    mtext(expression(paste("(b) ", italic(m), " = 100")), at=6, side=3, cex=1, las=1, line=0)
  }
  if (m == 160) {
    axis(2, seq(yMin, yMax, 1700), las=1, labels = FALSE)
    mtext(expression(paste("(c) ", italic(m), " = 160")), at=6, side=3, cex=1, las=1, line=0)
  }
}

# Examples of varying error levels
m = 100

for (e in c(10, 30, 50)) {

  detections.XY = generateSpeciesObservations(species, m - e)
  errors.XY = generateSpeciesObservationsWithErrors(species, e, e, temp, rain)
  
  nContour = 5
  cols = colorRampPalette(c("gray90", "gray40"))(nContour)

  image(unique(niche.XY[,1]), unique(niche.XY[,2]), fundNicheMatrix,
        xlim=c(xMin,xMax), xaxp=c(xMin,xMax,3), ylim=c(yMin,yMax), yaxp=c(yMin,yMax,5),
        breaks = seq(0, lambdaMax, lambdaMax/nContour), col=cols, main="",
        bty="n", xaxs="i", yaxs="i", xaxt="n", yaxt="n")
  contour(x=gridCoordsX, y=gridCoordsY, z=nz.climate, 
          levels = c(0.0000001, 0.0000005, 0.000001, 0.000005, 0.00001, 0.00005, 0.0001),
          col="black", lwd=0.25, drawlabels=FALSE, add=TRUE)
  points(detections.XY, pch=21, bg="#2c7bb6", cex=0.75, lwd=0.1)
  points(errors.XY, pch=21, bg="#d7191c", cex=0.75, lwd=0.1)
  axis(1, at = seq(xMin, xMax, 6), labels = FALSE)
  
  if (e == 10) {
    axis(2, seq(yMin, yMax, 1700), las=1)
    mtext("Rainfall (mm)", side=2, cex=1, line=2.5)
    mtext(expression(paste("(d) ", italic(m), " = 100, ", italic(e), " = 10 %")), at=6, side=3, cex=1, las=1, line=0)
  } 
  if (e == 30) {
    axis(2, seq(yMin, yMax, 1700), las=1, labels = FALSE)
    mtext(expression(paste("(e) ", italic(m), " = 100, ", italic(e), " = 30 %")), at=6, side=3, cex=1, las=1, line=0)
  }
  if (e == 50) {
    axis(2, seq(yMin, yMax, 1700), las=1, labels = FALSE)
    mtext(expression(paste("(f) ", italic(m), " = 100, ", italic(e), " = 50 %")), at=6, side=3, cex=1, las=1, line=0)
  }
}

# Examples of varying bias levels
for (b in c(20, 60, 100)) {

  detections.XY = generateSpeciesObservations(species, m - b)
  bias.XY = generateSpeciesObservationsWithBias(species, b, b, temp, rain)
  
  nContour = 5
  cols = colorRampPalette(c("gray90", "gray40"))(nContour)

  image(unique(niche.XY[,1]), unique(niche.XY[,2]), fundNicheMatrix,
        xlim=c(xMin,xMax), xaxp=c(xMin,xMax,3), ylim=c(yMin,yMax), yaxp=c(yMin,yMax,5),
        breaks = seq(0, lambdaMax, lambdaMax/nContour), col=cols, main="",
        bty="n", xaxs="i", yaxs="i", xaxt="n", yaxt="n")     
  contour(x=gridCoordsX, y=gridCoordsY, z=nz.climate, 
          levels = c(0.0000001, 0.0000005, 0.000001, 0.000005, 0.00001, 0.00005, 0.0001),
          col="black", lwd=0.25, drawlabels=FALSE, add=TRUE)
  
  points(detections.XY, pch=21, bg="#2c7bb6", cex=0.75, lwd=0.1)
  points(bias.XY, pch=21, bg="#d7191c", cex=0.75, lwd=0.1)
  axis(1, seq(xMin, xMax, 6))
  axis(2, seq(yMin, yMax, 1700), las=1, labels=FALSE)

  if (b == 20) {
    axis(2, seq(yMin, yMax, 1700), las=1)
    mtext(expression(paste("(g) ", italic(m), " = 100, ", italic(b), " = 20 %")), at=6, side=3, cex=1, las=1, line=0)
    
    legend(x=6, y=-1500, bty="n", pch=21, legend=c("Idealised sampling", "Contaminated sampling"), 
           pt.cex=0.75, pt.lwd=0.1, pt.bg=c("#2c7bb6", "#d7191c"), xjust=0.5, 
           title="Occurrences:", xpd=NA)
    
  }
  if (b == 60) {
    mtext(expression(paste("Temperature (", degree, "C)")), side=1, cex=1, line=2)
    mtext(expression(paste("(h) ", italic(m), " = 100, ", italic(b), " = 60 %")), at=6, side=3, cex=1, las=1, line=0)
    
    legend(x=6, y=-2000, bty="n", legend="New Zealand climate\nspace density contours", col="black", lwd=0.25, xjust=0.5, xpd=NA)
    
  }
  if (b == 100) {
    mtext(expression(paste("(i) ", italic(m), " = 100, ", italic(b), " = 100 %")), at=6, side=3, cex=1, las=1, line=0)
  }

}

par(oma=c(1.5,3,0,0.5))

image.plot(unique(niche.XY[,1]), unique(niche.XY[,2]), fundNicheMatrix,
           xlim=c(xMin,xMax), xaxp=c(xMin,xMax,3), ylim=c(yMin,yMax), yaxp=c(yMin,yMax,5),
           breaks = seq(0, lambdaMax, lambdaMax/nContour), col=cols, main="",
           bty="n", xaxs="i", yaxs="i", xaxt="n", yaxt="n",
           horizontal = TRUE, legend.only = TRUE,
           legend.shrink=1, legend.width=2,
           legend.mar=7,
           legend.lab=expression(paste("Fundamental niche (", lambda[italic(F)], ")")), legend.line=2.5,
           cex.axis = 1, cex.lab=1, add=TRUE)

#-------------------------------------------------------------------------------

dev.off()
embed_fonts("figure3.pdf", outfile="figure3.pdf")

#-------------------------------------------------------------------------------
